package com.coob.aframe.base;

import com.github.pagehelper.PageHelper;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

@Controller
public abstract class BaseController {
	protected Logger logger = Logger.getLogger(getClass());

	@Autowired
	protected HttpServletRequest request;
	@Autowired
	protected HttpServletResponse response;
	@Autowired
	protected HttpSession session;

	private int pageSize = 15;	//每页行数
	private int pageNum;		//当前页数

	public void setCurrentUser(LoginUser user){
		this.session.setAttribute("_SESSION_USER_INFO_", user);
	}

    public LoginUser getCurrentUser(){
    	LoginUser user = (LoginUser)this.session.getAttribute("_SESSION_USER_INFO_");
    	return user;
    }

    public void alert(String msg){
    	this.session.setAttribute("_resultMsg_Session_", msg);
    }
    
    
	/**
	 * 获取请求参数中所有的信息
	 * @return
	 */
	public Map<String, Object> getRequestParam() {
		Map<String, Object> res = new HashMap<String, Object>();
		Enumeration<?> temp = this.request.getParameterNames();
		if (null != temp) {
			while (temp.hasMoreElements()) {
				String en = (String) temp.nextElement();
				String value = this.request.getParameter(en);
				
				if( value != null && !value.equals("")){
					res.put(en, value);
				}
			}
		}
		return res;
	}
	
	
	/**
	 * 获取请求的参数，并保存到Session
	 *  用于列表页面查询
	 * @return
	 */
	public Map<String, Object> getRequestParamToSession() {
		Map<String, Object> res = new HashMap<String, Object>();
		Enumeration<?> temp = this.request.getParameterNames();
		if (null != temp) {
			while (temp.hasMoreElements()) {
				String en = (String) temp.nextElement();
				String value = this.request.getParameter(en);
				
				if( value != null && !value.equals("")){
					res.put(en, value);
				}
			}
		}
		this.session.setAttribute("_Session_Page_Params_", res);
		
		return res;
	}
	
	
	/**
	 * 将参数保存到session
	 * @param params
	 */
	public void setParamToSession(Map<String,Object> params){
		String entity = this.getClass().getSimpleName();
		this.session.setAttribute("_Session_Page_Params_"+entity, params);
	}
	
	
	/**
	 * 由session中获取保存参数对象
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public Map<String, Object> getSessionParam(){
		String entity = this.getClass().getSimpleName();
		Map<String, Object> res = (Map<String,Object>)this.session.getAttribute("_Session_Page_Params_"+entity);
		return res;
	}


	public void offsetPage(Integer pNum, Integer pSize){
		String entity = this.getClass().getSimpleName();

		if( pNum == null ) {
			pNum = (Integer) this.session.getAttribute("_Session_Page_PageNum_"+entity);
			System.out.println(" >> GetSession：" + pNum);
		}

		if( pNum == null ) {
			pNum = 1;
		}

		this.pageNum = pNum;
		if( pSize != null && pSize > 0 ) {
			this.pageSize = pSize;
		}

		this.session.setAttribute("_Session_Page_PageNum_"+entity, this.pageNum);

		int offset = (this.pageNum-1) * pageSize;

		PageHelper.offsetPage(offset, this.pageSize);
	}

	public void offsetPage(Integer pNum) {
		this.offsetPage(pNum, null);
	}
}
