package com.coob.aframe.fileCreate;

import java.io.File;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * 获取所有配置信息
 * @author Administrator
 *
 */
public class ConfigUtil {
	
	/**
	 * 读取数据并验证数据格式
	 * @param path
	 * @return
	 */
	public static ConfigBean getConfigInfo(String path){
		ConfigBean configBean = new ConfigBean();
		
		File file = new File(path);
		if( file.exists()){
			System.out.println(">>INFO: 开始读取配置文件 ...");
			try {
				DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
				DocumentBuilder builder = factory.newDocumentBuilder();
				Document document = builder.parse(file); 
				
				Element root = document.getDocumentElement();
				NodeList baseNodes = root.getChildNodes();
				
				boolean flag = false;
				
				for( int i=0; i<baseNodes.getLength(); i++){
					Node bNode = baseNodes.item(i);
					
					if( "datasource-config".equals(bNode.getNodeName())){
						//读取数据库连接配置
						flag = getConfigDataSource(configBean, bNode);
						if( !flag ){
							System.out.println("ERR: 数据库配置(datasource-config)检查不通过  << ");
							return null;
						}
						System.out.println("数据库配置读取完成...");
						
					} else if( "basefile-config".equals(bNode.getNodeName())){
						//基础dao、action配置
						flag = getConfigeBaseFile(configBean, bNode);
						if( !flag ){
							System.out.println("ERR: 基础类配置(basefile-config)检查不通过  << ");
							return null;
						}
						System.out.println("基础类配置读取完成...");
						
					} else if( "table-config".equals(bNode.getNodeName())){
						System.out.println(">>INFO: 开始读取数据库表配置 ...");
						NodeList tableNodes = bNode.getChildNodes();
						for( int j=0;j<tableNodes.getLength();j++ ){
							Node tNode = tableNodes.item(j);
							if( "table".equals(tNode.getNodeName())){
								flag = getConfigTable(configBean, tNode);
							}
						}
						System.out.println("数据库表配置读取完成...");
					}
				}
			} catch (Exception e) {
				System.out.println(">>ERR：数据格式有误<<");
				e.printStackTrace();
				System.out.println(">>ERR：数据格式有误<<");
			}
		}else{
			System.out.println(">>ERR 没有找到配置文件<<"+path);
			return null;
		}
		configBean.toString();
		return configBean;
	}
	
	/**
	 * 获取数据库配置信息
	 *  <datasource-config/>
	 * @param bean
	 * @return
	 */
	private static boolean getConfigDataSource(ConfigBean bean, Node node){
		if( node == null ) return false;
		
		NodeList nodeList = node.getChildNodes();
		if( nodeList == null || nodeList.getLength() == 0 )	return false;
		
		Node sourceNode = null;
		for( int i=0;i<nodeList.getLength();i++ ){	//寻找有效配置
			Node _dsNode = nodeList.item(i);
			if( "datasource".equals(_dsNode.getNodeName())){
				if( sourceNode == null ) sourceNode = _dsNode;	//默认第一个连接
				
				Node _node = _dsNode.getAttributes().getNamedItem("default");
				if( _node != null ){
					String _default = _node.getNodeValue();
					if( "1".equals(_default) || "true".equals(_default)){
						sourceNode = _dsNode;
						break;
					}
				}
			}
		}
		
		if( sourceNode != null ){
			//获取数据库连接配置
			bean.dataBaseType = sourceNode.getAttributes().getNamedItem("name").getNodeValue();
			NodeList _nodeList = sourceNode.getChildNodes();
			for(int i=0;i<_nodeList.getLength();i++ ){
				Node _node = _nodeList.item(i);
				if( "drive".equals(_node.getNodeName())){
					bean.driveClass = _node.getTextContent();
				} else if( "url".equals(_node.getNodeName())){
					bean.dataBaseUrl = _node.getTextContent();
				} else if( "username".equals(_node.getNodeName())){
					bean.username = _node.getTextContent();
				} else if( "password".equals(_node.getNodeName())){
					bean.password = _node.getTextContent();
				}
			}
			
			if( "mysql8x".equals(bean.dataBaseType)){
				//mysql8.x
				bean.dataBaseUrl = bean.dataBaseUrl + "?serverTimezone=GMT%2B8&useSSL=false&allowMultiQueries=true&useUnicode=true&characterEncoding=utf-8&zeroDateTimeBehavior=convertToNull";
			}
		}
		
		return bean.checkSource();
	}
	
	/**
	 * 获取基础类配置
	 *  <basefile-config/>
	 * @param bean
	 * @param node
	 * @return
	 */
	private static boolean getConfigeBaseFile(ConfigBean bean, Node node){
		if( node == null ) return false;
		
		NodeList nodeList = node.getChildNodes();
		for( int i=0;i<nodeList.getLength();i++ ){
			Node _node = nodeList.item(i);
			if( "baseAction".equals(_node.getNodeName())){
				bean.baseAction = _node.getTextContent();
				if( bean.baseAction != null && bean.baseAction.length()> 0 ){
					bean.baseActionName = bean.baseAction.substring(bean.baseAction.lastIndexOf(".")+1);
				}
			} else if( "rtnEntity".equals(_node.getNodeName())){
				bean.rtnEntity = _node.getTextContent();
				if( bean.rtnEntity != null && bean.rtnEntity.length()> 0 ){
					bean.rtnEntityName = bean.rtnEntity.substring(bean.rtnEntity.lastIndexOf(".")+1);
				}
			}
		}
		
		return bean.checkBase();
	}
	
	/**
	 * 读取数据库表配置
	 *  <table/>
	 * @param node
	 * @return
	 */
	private static boolean getConfigTable(ConfigBean bean, Node node){
		if( node == null ) return false;
		
		Node _n = null;
		_n = node.getAttributes().getNamedItem("create");
		if( _n != null ) {
			//判断是否生成
			String flag = _n.getNodeValue();
			if( flag != null && !flag.equals("1") && !flag.equals("true")){
				return true;
			}
		}
		
		ConfigTable configTable = new ConfigTable();
		
		_n = node.getAttributes().getNamedItem("name");
		if( _n != null ) configTable.tableName = _n.getNodeValue();
		
		_n = node.getAttributes().getNamedItem("javaEntiy");
		if( _n != null ) configTable.javaEntity = _n.getNodeValue();

		_n = node.getAttributes().getNamedItem("defaultMappingPath");
		if( _n != null ) configTable.defaultMappingPath = _n.getNodeValue();

		_n = node.getAttributes().getNamedItem("defaultJavaPath");
		if( _n != null ) configTable.defaultJavaPath = _n.getNodeValue();
		
		_n = node.getAttributes().getNamedItem("defaultJspPath");
		if( _n != null ) configTable.defaultJspPath = _n.getNodeValue();
		
		_n = node.getAttributes().getNamedItem("namespace");
		if( _n != null ) configTable.namespace = _n.getNodeValue();
		
		_n = node.getAttributes().getNamedItem("action");
		if( _n != null ) configTable.action = _n.getNodeValue();
		
		_n = node.getAttributes().getNamedItem("sqlmapsConfig");
		if( _n != null ) {
			String s = _n.getNodeValue();
			if( !"0".equals(s) && !"false".equals(s)){
				configTable.sqlmapsConfig = _n.getNodeValue();
			}
		}
		
		_n = node.getAttributes().getNamedItem("applicationConfig");
		if( _n != null ) {
			String s = _n.getNodeValue();
			if( !"0".equals(s) && !"false".equals(s)){
				configTable.applicationConfig = _n.getNodeValue();
			}
		}
		
		_n = node.getAttributes().getNamedItem("strutsConfig");
		if( _n != null ) {
			String s = _n.getNodeValue();
			if( !"0".equals(s) && !"false".equals(s)){
				configTable.strutsConfig = _n.getNodeValue();
			}
		}
		
		//判断文件生成配置
		NodeList nodeList = node.getChildNodes();
		for( int i=0; i<nodeList.getLength(); i++ ){
			Node fileNode = nodeList.item(i);
			
			//读取文件配置
			if( "file-create".equals(fileNode.getNodeName())){
				NodeList fileNodeList = fileNode.getChildNodes();
				if( fileNodeList != null && fileNodeList.getLength() > 0 ) {
					for( int j=0;j<fileNodeList.getLength();j++ ){
						Node _node = fileNodeList.item(j);
						if("file".equals(_node.getNodeName())){
							String t = null;
							String n = null;
							String p = null;
							String c = null;
							
							Node nt = _node.getAttributes().getNamedItem("type");
							Node nn = _node.getAttributes().getNamedItem("name");
							Node np = _node.getAttributes().getNamedItem("path");
							Node nc = _node.getAttributes().getNamedItem("create");
							
							if( nt != null ) t = nt.getNodeValue();
							if( nn != null ) n = nn.getNodeValue();
							if( np != null ) p = np.getNodeValue();
							if( nc != null ) c = nc.getNodeValue();
							
							if( t == null ) continue;
							if( "sql".equals(t)){
								configTable.sqlName = n;
								configTable.sqlPath = p;
								configTable.sqlFlag = formatFlag(c);
							} else if( "domain".equals(t)){
								configTable.domainName = n;
								configTable.domainPath = p;
								configTable.domainFlag = formatFlag(c);
							} else if( "dao".equals(t)){
								configTable.daoName = n;
								configTable.daoPath = p;
								configTable.daoFlag = formatFlag(c);
							} else if( "service".equals(t)){
								configTable.serviceName = n;
								configTable.servicePath = p;
								configTable.serviceFlag = formatFlag(c);
							} else if( "action".equals(t)){
								configTable.actionName = n;
								configTable.actionPath = p;
								configTable.actionFlag = formatFlag(c);
							} else if( "list".equals(t)){
								configTable.listName = n;
								configTable.listPath = p;
								configTable.listFlag = formatFlag(c);
							} else if( "add".equals(t)){
								configTable.addName = n;
								configTable.addPath = p;
								configTable.addFlag = formatFlag(c);
							} else if( "edit".equals(t)){
								configTable.editName = n;
								configTable.editPath = p;
								configTable.editFlag = formatFlag(c);
							} else if( "view".equals(t)){
								configTable.viewName = n;
								configTable.viewPath = p;
								configTable.viewFlag = formatFlag(c);
							}
						}
					}
				}
			} 
		}
		
		bean.addTable(configTable);
		
		return true;
	}
	
	private static boolean formatFlag(String flag){
		if( flag == null ){
			return true;
		}else {
			flag = flag.toLowerCase();
			if( flag.equals("1") || flag.equals("true")){
				return true;
			}else {
				return false;
			}
		}
	}
}
