package com.coob.aframe.fileCreate;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 连接数据库并查询表结构
 */
public class ConnDatabase {
	
	public ConnDatabase(){ }

	/**
	 * 获取数据库表信息
	 * @param config
	 */
	public void getTableInfo(ConfigBean config){
		try{
			Connection conn = this.getConnection(config);
			this.getTables(conn, config);
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	/**
	 * 获取数据库连接
	 * @param config
	 * @return
	 */
	private Connection getConnection(ConfigBean config){
		Connection conn = null;
		try {
			Class.forName(config.driveClass);
			System.out.println("加载驱动完成");
			conn = DriverManager.getConnection(config.dataBaseUrl, config.username, config.password );
			System.out.println(conn);
		} catch (SQLException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} 
		return conn;
	}
	
	/**
	 * 查询数据库表
	 * @param conn
	 * @param config
	 */
	private void getTables(Connection conn, ConfigBean config){
		try {
			for( ConfigTable table : config.tableList ){
				getCols(conn,table, config.dataBaseType);
			}
			conn.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 查询表结构
	 * @param conn
	 * @param dbtype
	 * @param table
	 * @return
	 */
	private void getCols(Connection conn, ConfigTable table, String dbtype){
		List<ColumnBean> cloList = new ArrayList<ColumnBean>();
		
		dbtype = dbtype.toLowerCase();
		
		String sql = null;
		if( dbtype.indexOf("mysql") > -1 ){
			sql = "select * from "+table.tableName+" limit 1";
		}else if( dbtype.equals("sqlserver")){
			sql = "select top 1 * from "+table.tableName;
		}else if( dbtype.equals("oracle")){
			sql = "select * from "+table.tableName + " where ROWNUM <=1";
		}
		System.out.println(sql);
		try {
			Statement st = conn.createStatement();
			ResultSet rs = st.executeQuery(sql);
			ResultSetMetaData rsmd = rs.getMetaData();
			int colcount = rsmd.getColumnCount();

			// <列名,注释> 集合，目前仅支持Mysql
			Map<String,String> columnComments = new HashMap<String,String>();
			if( dbtype.equals("mysql")){
				ResultSet rsfull = st.executeQuery("show full columns from " + table.tableName);
				while (rsfull.next()) {
					String _comment = rsfull.getString("Comment");
					if( _comment != null && _comment.length() > 0 ){
						columnComments.put(rsfull.getString("Field").toUpperCase(),rsfull.getString("Comment"));
					}
				}
			}

			int clen = 0;
			int blen = 0;
			for(int i=1;i<=colcount;i++){
				String colName = rsmd.getColumnName(i).toUpperCase();
				String colType = rsmd.getColumnTypeName(i).toUpperCase();
				String colcomment = columnComments.get(colName);
				if( colcomment == null ){
					colcomment = "";
				}

				ColumnBean col = new ColumnBean(colName, colType, colcomment);
				
				if( clen < col.getColLen() ) clen = col.getColLen();
				if( blen < col.getBeanLen() ) blen = col.getBeanLen();
				
				if( col.getBeanType().equals("Date")) table.columnHasDate = true;
				if( col.getBeanType().equals("BigDecimal")) table.columnHasDecimal = true;

				cloList.add(col);
			}
			
			table.colMaxLength = clen;
			table.domMaxLength = blen;
			
			rs.close();
			st.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		table.columnList = cloList;
	}

}
