package com.coob.mis.cx.controller;

import com.coob.aframe.base.BaseController;
import com.coob.mis.cx.domain.MisHouse;
import com.coob.mis.cx.service.MisHouseService;

import com.github.pagehelper.PageInfo;
import javax.annotation.Resource;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.ui.Model;
import java.util.List;
import java.util.Map;
import java.util.HashMap;

@RequestMapping("/mis/cx/house")
@Controller
public class MisHouseController extends BaseController {
	@Resource
	private MisHouseService misHouseService;


	/**
	 * 查询列表
	 * @param model
	 * @return
	 */
	@RequestMapping(value = "/doList", method = {RequestMethod.GET, RequestMethod.POST})
	public String doList(Model model) {
		Map<String,Object> params = new HashMap<String,Object>();

		this.setParamToSession(params);

		super.offsetPage(1);
		List<MisHouse> list = this.misHouseService.selectByParams(params);
		PageInfo<MisHouse> pageInfo = new PageInfo<>(list);

		model.addAttribute("pageInfo", pageInfo);

		return "pages/mis/cx/house/list";
	}


	/**
	 * 分页
	 * @param pageNum
	 * @param model
	 * @return
	 */
	@RequestMapping(value = "/doPage")
	public String doPage(Integer pageNum, Model model) {
		Map<String,Object> params = this.getSessionParam();

		super.offsetPage(pageNum);
		List<MisHouse> list = this.misHouseService.selectByParams(params);
		PageInfo<MisHouse> pageInfo = new PageInfo<>(list);

		model.addAttribute("pageInfo", pageInfo);

		return "pages/mis/cx/house/list";
	}


	/**
	 * 进入新增页面
	 * @param model
	 * @return
	 */
	@RequestMapping(value = "/toAdd")
	public String toAdd(Model model) {
		MisHouse misHouse = new MisHouse();
		model.addAttribute("misHouse", misHouse);

		return "pages/mis/cx/house/add";
	}


	/**
	 * 保存新增信息
	 * @param misHouse
	 * @param model
	 * @return
	 */
	@RequestMapping(value = "/doAdd")
	public String doAdd(MisHouse misHouse, Model model){
		int flag = this.misHouseService.insert(misHouse);

		this.alert("信息保存成功");

		return "redirect:doList";
	}


	/**
	 * 进入修改页面
	 * @param id
	 * @param model
	 * @return
	 */
	@RequestMapping(value = "/toEdit")
	public String toEdit(Integer id, Model model) {

		MisHouse misHouse = this.misHouseService.selectById(id);
		model.addAttribute("misHouse", misHouse);

		return "pages/mis/cx/house/edit";
	}


	/**
	 * 保存修改信息
	 * @param misHouse
	 * @param model
	 * @return
	 */
	@RequestMapping(value = "/doEdit")
	public String doEdit(MisHouse misHouse, Model model) {
		int flag = this.misHouseService.update(misHouse);
		this.alert("信息保存成功");

		return "pages/mis/cx/house/edit";
	}

	/**
	 * 删除信息
	 * @param id
	 * @param model
	 * @return
	 */
	@RequestMapping(value = "/doDel")
	public String doDel(Integer id, Model model) {
		int flag = this.misHouseService.deleteById(id);
		this.alert("信息删除成功");

		return "redirect:doPage";
	}

	/**
	 * 查看详细信息
	 * @param id
	 * @param model
	 * @return
	 */
	@RequestMapping(value = "/toView")
	public String toView(Integer id, Model model) {
		MisHouse misHouse = this.misHouseService.selectById(id);
		model.addAttribute("misHouse", misHouse);

		return "pages/mis/cx/house/view";
	}


}
