package com.coob.zhfg.system.controller;

import com.coob.aframe.base.BaseController;
import com.coob.aframe.base.LoginUser;
import com.coob.aframe.util.DESPwd;
import com.coob.aframe.util.DateUtil;
import com.coob.aframe.util.MD5Util;
import com.coob.zhfg.system.domain.Menu;
import com.coob.zhfg.system.domain.User;
import com.coob.zhfg.system.service.RoleService;
import com.coob.zhfg.system.service.UserService;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import javax.annotation.Resource;
import java.util.List;


/**
 * @Author: Gao
 * @Date: 2024/1/30/17:41
 * @Description: 用户登录
 */
@Controller
@RequestMapping("/")
public class LoginController extends BaseController {
    @Resource
    private UserService userService;
    @Resource
    private RoleService roleService;


    /**
     * 登陆页面
     * @return
     */
    @PostMapping("/login")
    public String login(String account, String password, Model model) {
        System.out.println( DateUtil.curT() + " > 用户登录：account=" + account);

        //验证账号密码输入
        if( account == null || account.length() == 0 || account == null || account.length() == 0){
            return "login";
        }

        //判断账户
        User user = this.userService.selectByDlzh(account);
        if( user == null ){
            model.addAttribute("resultMsg", "账户不存在");
            return "login";
        }

        //判断密码
        String pwd = password;
        if( user.getMmfs() == 1 ){
           pwd = DESPwd.encrypt(password);
            System.out.println(" > 加密方式：DES, PWD=" + pwd);

        } else if( user.getMmfs() == 5 ) {
            pwd = MD5Util.MD5(password);
            System.out.println(" > 加密方式：MD5, PWD=" + pwd);
        }

        if( !pwd.equals(user.getDlmm())) {
            model.addAttribute("resultMsg", "登录密码错误");
            return "login";
        }

        LoginUser loginUser = new LoginUser();
        loginUser.setUserInfo(user);

        Integer bmid = user.getBmid();
//        if( bmid != null ) {
//            Depart depart = this.departService.selectById(bmid);
//            if( depart != null ) {
//                loginUser.setDepartInfo(depart);
//            }
//        }

        //建立Session
        this.session = this.request.getSession(true);
        super.setCurrentUser(loginUser);

        //查询操作菜单
        String menuStr =  this.menuStr(loginUser);

        model.addAttribute("loginUser", loginUser);
        model.addAttribute("menuStr", menuStr);

        return "main";
    }


    /**
     * 获取登录账户权限菜单
     * @param user
     * @return
     */
    private String menuStr(LoginUser user){
        StringBuffer menuStr = new StringBuffer();
        List<Menu> mList = null;

        if (user.getUserType() == 1) {
            // 超级管理员获取全部
            mList = this.roleService.selectMenuList();
        } else {
            mList = this.roleService.selectMenuListByUserId(user.getUserId());
        }

        if( mList != null ){
            for( Menu m : mList){
                if( m.getOrders() == 2 ){
                    menuStr.append("<div title= \""+ m.getMenuName()+"\" data-options=\"iconCls:'fa fa-arrow-right'\" style=\"padding: 0px;overflow:auto;\">");
                    menuStr.append("<ul class=\"easyui-tree\">");
                    for( Menu c : mList ){
                        if( c.getParentId().intValue() == m.getId().intValue() ){
                            menuStr.append( "<li><div style=\"width:100%;hight:100%\"><a href=\"javascript:addTab('"+c.getMenuName()+"','"+c.getUrl()+"','fa fa-plus')\">"
                                    +c.getMenuName()+"</a></div></li> ");
                        }
                    }
                    menuStr.append("</ul></div>");
                }
            }
        }

        return menuStr.toString();
    }



}
