
function FileUpload(obj){
	var fileId   = obj.fileId;		//文件input标签id
	var divId    = obj.divId;		//提示信息DivId
	var url      = obj.url;			//后台接收路径
	var fileType = obj.fileType;	//文件类型(后缀名,逗号分隔)
	var compress = obj.compress;	//是否压缩 0/false不压缩（默认）， 1/true 压缩 （必须是图片）
	var maxSize  = obj.maxSize;		//压缩上线大小KB（必须 compress=1)  0/null 不压缩
	var data     = obj.data;		//附加参数
	var auto     = obj.auto;		//是否自动上传 0/false不自动上传（默认）， 1/true自动上传
	var callback = obj.callback;	//上传成功后的回调方法
	
	if( compress ){
		//未设置文件限定大小，不压缩
    	if( maxSize == null || maxSize == 0){
    		compress = false;
    	} else {
    		maxSize = maxSize * 1024;	//KB转为B
    	}
    }
	
	var divObj = null;	//提示div对象
	var files = null;	//由input标签中提取的文件列表
	var curFileNo = -1;	//当前上传的文件序号
	var xhr = null;		//XMLHttpRequest 对象
	var fileTypeCheck = true;	//格式验证是否通过，默认通过
	
	this.setData = function(dat){
		data = dat;
	};
	
	/**
	 * 选择文件时调用方法
	 */
	this.select = function(){
		if( document.getElementById(fileId).value == ''){
			return false;
		}
		
		//获取选择的文件
		files = document.getElementById(fileId).files;
		divObj = document.getElementById(divId);
		divObj.innerHTML = '';
		
		//验证文件格式
		fileTypeCheck = true;
		if( fileType != null ){
			var fType = fileType.split(',');
			for(var ft in fType ){
				fType[ft] = fType[ft].replace(/^\s+|\s+$/gm,'');
			}
			
			for( var i=0; i<files.length; i++ ){
				var fName = files[i].name;
				var index = fName.lastIndexOf(".")+1;
				if( index > 1 ){
					var _flag = false;
					var suffix = fName.substring(index).toLowerCase();
					for( var ft in fType ){
						if( fType[ft] == suffix ){
							_flag = true;
							break;
						}
					}
					if( _flag == false ){
						divObj.innerHTML = divObj.innerHTML + fName + ' 文件格式错误<br/>';
						fileTypeCheck = false;
					}
				} else {
					divObj.innerHTML = divObj.innerHTML + fName + ' 文件格式错误<br/>';
					fileTypeCheck = false;
				}
			}
		}
		//文件格式验证不通过
		if( fileTypeCheck == false ){
			return false;
		}
		
		//显示选择的文件并添加进度条等元素
		for( var i=0; i<files.length; i++ ){
			var fName = files[i].name;
			
			var div = document.createElement("div");
			div.setAttribute('id', 'fileNameDiv'+i);
			div.setAttribute('style',"display:block;");
			divObj.appendChild(div);
			
			var nmSpan = document.createElement("span");
			nmSpan.setAttribute('id', 'fileNameSpan'+i);
			nmSpan.setAttribute('style',"display:inline-block; padding-right:8px;");
			div.appendChild(nmSpan);
			nmSpan.innerHTML = fName;
			
			var progress = document.createElement("progress");
			progress.setAttribute('id', 'progressBar'+i);
			progress.setAttribute('value', 0);
			progress.setAttribute('max', 100);
			progress.setAttribute('style', "width:150px; display:none;");
			div.appendChild(progress);
			
			var rsSpan = document.createElement("span");
			rsSpan.setAttribute('id', 'fileResultSpan'+i);
			rsSpan.setAttribute('style', "display:inline-block;");
			div.appendChild(rsSpan);
		}
		
		if( auto ){
			curFileNo = -1;
			for( var i=0; i<files.length; i++ ){
				//自动上传文件
				uploadOne();
			}
		}
	};
	
	
	this.upload = function(){
		if( files != null && files.length > 0 ){
			curFileNo = -1;
			for( var i=0; i<files.length; i++ ){
				//自动上传文件
				uploadOne();
			}
		}
	};
	
	
	/**
	 * 上传单个文件，非页面直接调用方法
	 * @returns {Boolean}
	 */
	function uploadOne() {
		curFileNo ++;
		if( files == null || curFileNo >= files.length ){
			document.getElementById(fileId).value = '';
			return false;
		} 
		
		var bar = document.getElementById('progressBar'+curFileNo);
		bar.style.display = 'inline-block';
		var fileObj = files[curFileNo];
		
	    var form = new FormData(); // FormData 对象
	    
	    //添加参数
	    if( data != null)
	    	for(var pKey in data){
	    		var pVal = data[pKey];
	    		form.append(pKey, pVal);
	    	}
	    
	    //判断是否压缩文件
	    var compressFlag = compress;
	    if( compress ){
	    	if( fileObj.size <= maxSize ){
	    		//不超过限定大小，不需要压缩
	    		compressFlag = false;
	    		
	    	} else {
		    	//按设置需要压缩文件，判断文件是否为图片（仅图片支持压缩）
		    	var fName = fileObj.name;
		 		var _index = fName.lastIndexOf(".")+1;
		 		if( _index > 1 ){
		 			var suffix = fName.substring(_index).toLowerCase();
		 			if( suffix != 'png' && suffix != 'jpg' && suffix != 'jpeg' ){
			 			//非图片文件，不压缩
		 				compressFlag = false;
			 		}
		 		} else {
		 			//未知格式文件，不压缩
		 			compressFlag = false;
		 		}
	    	}
	    }
	    
	    if( compressFlag ){
	    	//压缩文件
	    	photoCompress(fileObj, form);
	    	
	    } else {
	    	//不压缩，直接发送文件对象
	    	form.append("file", fileObj);
	    	xhr = new XMLHttpRequest();  		// XMLHttpRequest 对象
		    xhr.open("post", url, true); 		//post方式，url为服务器请求地址，true 该参数规定请求是否异步处理。
		    xhr.onload = uploadComplete; 		//请求完成
		    xhr.onerror = uploadFailed; 		//请求失败

		    xhr.upload.onprogress = progressFunction;		//【上传进度调用方法实现】
		    //this.xhr.upload.onloadstart = function(){//上传开始执行方法
				//ot = new Date().getTime();   //设置上传开始时间
				//oloaded = 0;	//设置上传开始时，以上传的文件大小为0
			//};

			xhr.send(form); //开始上传，发送form数据
	    }
	}
	
	
	/**
	 * 上传成功响应
	 */
	function uploadComplete(evt) {
		document.getElementById("fileResultSpan" + curFileNo).innerHTML = '上传成功';
		if( callback !=  null ){
			callback(evt.target.responseText);
		}
		uploadOne();	//上传下一个文件
	}


	/**
	 * 上传失败
	 */
	function uploadFailed(evt) { 
		document.getElementById("fileResultSpan" + curFileNo).innerHTML = '上传失败';
		uploadOne();	//上传下一个文件
	}
	

	/**
	 * 取消上传
	 */
	this.cancle = function(){
		xhr.abort();
	};


	/**
	 * 上传进度实现方法，上传过程中会频繁调用该方法
	 */
	function progressFunction(evt) {
		var progressBar = document.getElementById("progressBar" + curFileNo);
		var percentageDiv = document.getElementById("fileResultSpan" + curFileNo);
		// event.total是需要传输的总字节，event.loaded是已经传输的字节。如果event.lengthComputable不为真，则event.total等于0
		if (evt.lengthComputable) {
			progressBar.max = evt.total;
			progressBar.value = evt.loaded;
			percentageDiv.innerHTML = Math.round(evt.loaded / evt.total * 100) + "%";
		}
		
		//计算上传时间代码见最后注释，暂时不用
		
	}
	
	
	/**
	 * 文件压缩上传
	 */
	function photoCompress(file, form){
		var ready = new FileReader();
		ready.readAsDataURL(file);
		ready.onload=function(){
            var res = this.result;
            
            var img = new Image();
            img.src = res;
            img.onload = function(){
                //生成canvas
                var canvas = document.createElement('canvas');
                var ctx = canvas.getContext('2d');
                canvas.width = img.width;
                canvas.height = img.height;
		        
                ctx.clearRect(0, 0, canvas.width, canvas.height);
		        ctx.drawImage(img, 0, 0, img.width, img.height);
		        
		        var quality = getCompressRate(maxSize, file.size);	//压缩比例
		        var dataUrl = canvas.toDataURL('image/jpeg', quality);
               
                //发送文件
		        var bl = convertBase64UrlToBlob(dataUrl);
                form.append(file.name, bl); // 文件对象
                xhr = new XMLHttpRequest();  // XMLHttpRequest 对象
                xhr.open("post", url, true); //post方式，url为服务器请求地址，true 该参数规定请求是否异步处理。
                xhr.onload = uploadComplete; //请求完成
                xhr.onerror =  uploadFailed; //请求失败

                xhr.upload.onprogress = progressFunction;	//【上传进度调用方法实现】
                xhr.send(form); //开始上传，发送form数据
            };
        };
    }
	
	
	/**
	 * 计算压缩比例
	 */
	function getCompressRate(allowMaxSize, fileSize){ //计算压缩比率，size单位为MB
		var compressRate = 1;
		
		if(fileSize/allowMaxSize > 8){
			compressRate = 0.1;
		} else if(fileSize/allowMaxSize > 4){
			compressRate = 0.2;
		} else if(fileSize/allowMaxSize >3){
			compressRate = 0.4;
		} else if(fileSize/allowMaxSize >2){
			compressRate = 0.6;
		} else if(fileSize > allowMaxSize){
			compressRate = 0.8;
		} else{
			compressRate = 1;
		}
		return compressRate;
	}
	
	
	/**
     * 将以base64的图片url数据转换为Blob
     * @param urlData
     *            用url方式表示的base64图片数据
     */
    function convertBase64UrlToBlob(urlData){
        var arr = urlData.split(','), mime = arr[0].match(/:(.*?);/)[1],
            bstr = atob(arr[1]), n = bstr.length, u8arr = new Uint8Array(n);
        while(n--){
            u8arr[n] = bstr.charCodeAt(n);
        }
        return new Blob([u8arr], {type:mime});
    }
}


/*
 * 前端页面样例
<!DOCTYPE html>
<html>
<head>
    <meta charset="UTF-8">
    <script type="text/javascript" src="/wxzj/js/FileUpload.js"></script>
    <script type="text/javascript">
        var fUpload = new FileUpload({
    		fileId: 'file',
    		divId: 'shuomingDiv',
    		url: "/wxzj/fileRegTestServlet",
    		fileType: "png, jpg, jpeg",
    		compress: true,
    		maxSize: 500,
    		data: {"appId":1},
    		auto: false,
    		callback: uploadOk
    	});
        function selectFiles(fileId, divId){
        	fUpload.select();
        }
        function upladFiles() {
        	fUpload.upload();
        }
        function cancleUpload(){
        	fUpload.cancle();
        }
        function uploadOk(t){
        	alert(t);
        }
    </script>
</head>
<body>
<input type="file" id="file" name="myfile" multiple="multiple" onchange="selectFiles()" accept="image/x-png, image/jpg, image/jpeg, image/gif"/>
<input type="button" onclick="upladFiles()" value="上传" />
<input type="button" onclick="cancleUpload()" value="取消" />
<div id="shuomingDiv" style="font-size: 12px; font-weight: normal; color: #999999; line-height:28px;"></div>
</body>
</html>
 */




/* 下面是计算上传时间代码
var time = document.getElementById("time");
var nt = new Date().getTime();//获取当前时间
var pertime = (nt-ot)/1000; //计算出上次调用该方法时到现在的时间差，单位为s
ot = new Date().getTime(); //重新赋值时间，用于下次计算
var perload = evt.loaded - oloaded; //计算该分段上传的文件大小，单位b
oloaded = evt.loaded;//重新赋值已上传文件大小，用以下次计算
//上传速度计算
var speed = perload/pertime;//单位b/s
var bspeed = speed;
var units = 'b/s';//单位名称
if(speed/1024>1){
    speed = speed/1024;
    units = 'k/s';
}
if(speed/1024>1){
    speed = speed/1024;
    units = 'M/s';
}
speed = speed.toFixed(1);
//剩余时间
var resttime = ((evt.total-evt.loaded)/bspeed).toFixed(1);
time.innerHTML = '，速度：'+speed+units+'，剩余时间：'+resttime+'s';

if(bspeed==0) time.innerHTML = '上传已取消';
*/