/**
 * 除法函数，用来得到精确的除法结果  
 * 说明：javascript的除法结果会有误差，在两个浮点数相除的时候会比较明显。这个函数返回较为精确的除法结果。 
 * @param arg1
 * @param arg2
 * @returns arg1/arg2的精确结果 
 */
function accDiv(arg1,arg2){ 
	var t1=0, t2=0, r1, r2; 
	try {
		t1=arg1.toString().split(".")[1].length;
	}catch(e){} 
	try{
		t2=arg2.toString().split(".")[1].length;
	}catch(e){} 
	with(Math){ 
		r1=Number(arg1.toString().replace(".",""));
		r2=Number(arg2.toString().replace(".",""));
		return (r1/r2)*pow(10,t2-t1); 
	} 
} 


/**
 * 乘法函数，用来得到精确的乘法结果 
 * 说明：javascript的乘法结果会有误差，在两个浮点数相乘的时候会比较明显。这个函数返回较为精确的乘法结果。 
 * @param arg1
 * @param arg2
 * @returns arg1*arg2的精确结果 
 */
function accMul(arg1,arg2){ 
	var m=0,s1=arg1.toString(),s2=arg2.toString(); 
	try{
		m+=s1.split(".")[1].length;
	}catch(e){} 
	try{
		m+=s2.split(".")[1].length;
	}catch(e){} 
	return Number(s1.replace(".",""))*Number(s2.replace(".",""))/Math.pow(10,m);
}


/**
 * 加法函数，用来得到精确的加法结果
 * 说明：javascript的加法结果会有误差，在两个浮点数相加的时候会比较明显。这个函数返回较为精确的加法结果。 
 * @param arg1
 * @param arg2
 * @returns arg1+arg2的精确结果 
 */
function accAdd(arg1,arg2){ 
	var r1,r2,m; 
	try{
		r1=arg1.toString().split(".")[1].length;
	}catch(e){
		r1=0;
	} 
	try{
		r2=arg2.toString().split(".")[1].length;
	}catch(e){
		r2=0;
	} 
	m=Math.pow(10,Math.max(r1,r2));
	
	v = (arg1*m+arg2*m)/m * 10000;
	return Math.round(v)/10000;
} 


/**
 * 减法函数，用来得到精确的减法结果 
 * 说明：javascript的减法结果会有误差，在两个浮点数相加的时候会比较明显。这个函数返回较为精确的减法结果。 
 * @param arg1
 * @param arg2
 * @returns arg1-arg2的精确结果 
 */
function accSubtr(arg1, arg2){
	var r1, r2, m, n;
	try{r1 = arg1.toString().split(".")[1].length;}catch(e){r1=0;}
	try{r2 = arg2.toString().split(".")[1].length;}catch(e){r2=0;}
	m = Math.pow(10,Math.max(r1,r2));
	//动态控制精度长度
	n=(r1>=r2)?r1:r2;
	return ((arg1*m-arg2*m)/m).toFixed(n);
} 


/**
 * 保留小数位，小数位四舍五入，小数位不足的补0
 * @param number 原数据
 * @param n 小数位数（0-20之间）
 * @returns
 */
function toFixed(number, n) {
	// 如果 digits 参数太小或太大。0 到 20（包括）之间的值不会引起 RangeError。实现环境（implementations）也可以支持更大或更小的值
	if (n > 20 || n < 0) {
		throw new RangeError('toFixed() digits argument must be between 0 and 20');
	}
	// 如果该方法在一个非Number类型的对象上调用
	if (isNaN(number)) {
		throw new TypeError(number + '.toFixed() is not a function');
	}
	// 如果数值大于 1e+21，该方法会简单调用 Number.prototype.toString()并返回一个指数记数法格式的字符串
	if (number >= Math.pow(10, 21)) {
		return number.toString();
	}
	// 如果忽略该参数，则默认为 0，进行四舍五入，不包括小数部分
	if (n === undefined || n === 0) {
		return Math.round(number).toString();
	}

	var result = number.toString();
	var arr = result.split('.');
	// 整数情况
	if (arr.length < 2) {
		result += '.';
		for (var i = 0; i < n; i ++) {
			result += '0';
		}
		return result;
	}

	// 小数情况（原位数 <= 需求位数）
	var integer = arr[0];
	var decimal = arr[1];
	if (decimal.length === n) {
		return result;
	}
	if (decimal.length < n) {
		for (var i = 0; i < n - decimal.length; i++) {
			result += '0';
		}
		return result;
	}
	// 小数情况（原位数 > 需求位数，需四舍五入）
	result = integer + '.' + decimal.substring(0, n);
	var last = decimal.substring(n, n+1); 
	if (parseInt(last, 10) >= 5) {
		var multiple = Math.pow(10, n);
		result = ((Math.round(parseFloat(result) * multiple) + 1) / multiple).toString(); // 使用 Math.round 防止结果无限小数
		// 0 补足小数
		decimal = result.split('.')[1];
		if( decimal == null ){
			decimal = '';
			result = result + ".";
		}
		if (decimal.length < n) {
			for (var i = 0; i < n - decimal.length; i++) {
				result += '0';
			}
			return result;
		}
	}
	return result;
}
